import type OpenAI from "openai";
type OpenAIFunction = OpenAI.Chat.ChatCompletionCreateParams.Function;
/** Types representing the OpenAI function definitions. While the OpenAI client
 * library does have types for function definitions, the properties are just
 * Record<string, unknown>, which isn't very useful for type checking this
 * formatting code. */
export interface FunctionDef extends Omit<OpenAIFunction, "parameters"> {
    readonly name: string;
    readonly description?: string;
    readonly parameters: ObjectProp;
}
export interface ObjectProp {
    readonly type: "object";
    readonly properties?: Record<string, Prop>;
    readonly required?: string[];
}
export interface AnyOfProp {
    readonly anyOf: Prop[];
}
export type Prop = {
    description?: string;
} & (AnyOfProp | ObjectProp | {
    type: "array";
    items?: Prop;
} | {
    type: "integer" | "number";
    minimum?: number;
    maximum?: number;
    enum?: number[];
} | {
    type: "string";
    enum?: string[];
} | {
    type: "boolean";
} | {
    type: "null";
});
export declare function isAnyOfProp(prop: Prop): prop is AnyOfProp;
/** When OpenAI use functions in the prompt, they format them as TypeScript
 * definitions rather than OpenAPI JSON schemas. This function converts the JSON
 * schemas into TypeScript definitions. */
export declare function formatFunctionDefinitions(functions: FunctionDef[]): string;
/** Format just the properties of an object (not including the surrounding
 * braces) */
export declare function formatObjectProperties(obj: ObjectProp, indent: number): string;
/** Format a single property type */
export declare function formatType(param: Prop, indent: number): string;
export {};
//# sourceMappingURL=functions.d.ts.map