import type OpenAI from "openai";
import type { FunctionDef } from "./functions.js";
type Message = OpenAI.Chat.ChatCompletionMessageParam;
type OpenAIFunction = OpenAI.Chat.ChatCompletionCreateParams.Function;
type FunctionCall = OpenAI.Chat.ChatCompletionFunctionCallOption;
/** OpenAI prompt data */
export interface Prompt {
    /** OpenAI chat messages */
    readonly messages: Message[];
    readonly function_call?: FunctionCall | "auto" | "none";
    /** OpenAI function definitions */
    readonly functions?: OpenAIFunction[];
}
/**
 * Estimates the number of tokens a prompt will use.
 * @returns An estimate for the number of tokens the prompt will use.
 */
export declare function promptTokensEstimate({ messages, functions, function_call, }: Prompt): number;
/**
 * Count the number of tokens in a string.
 * @param s The string to count tokens in
 * @returns The number of tokens in the string
 */
export declare function stringTokens(s: string): number;
/**
 * Estimate the number of tokens a message will use. Note that using the message
 * within a prompt will add extra tokens, so you might want to use
 * `promptTokensEstimate` instead.
 * @param message An OpenAI chat message
 * @returns An estimate for the number of tokens the message will use
 */
export declare function messageTokensEstimate(message: Message): number;
/**
 * Estimate the number of tokens a function definition will use. Note that using
 * the function definition within a prompt will add extra tokens, so you might
 * want to use `promptTokensEstimate` instead.
 * @param funcs An array of OpenAI function definitions
 * @returns An estimate for the number of tokens the function definitions will use
 */
export declare function functionsTokensEstimate(funcs: FunctionDef[]): number;
export {};
//# sourceMappingURL=token-counts.d.ts.map